@@ -421,13 +421,13 @@ zip_flush (GsfOutfileZip *zip)
 
 	do {
 		zret = deflate (zip->stream, Z_FINISH);
-		if (zret == Z_OK) {
-			/*  In this case Z_OK means more buffer space
-			    needed  */
+		if (zret == Z_OK || (zret == Z_BUF_ERROR && zip->stream->avail_out == 0)) {
+			/*  In this case Z_OK or Z_BUF_ERROR means more buffer 
+			    space is needed  */
 			if (!zip_output_block (zip))
 				return FALSE;
 		}
-	} while (zret == Z_OK);
+	} while (zret == Z_OK || zret == Z_BUF_ERROR);
 	if (zret != Z_STREAM_END)
 		return FALSE;
 	if (!zip_output_block (zip))
@@ -499,7 +499,18 @@ zip_close_stream (GsfOutput *output)
 	}
 	zip->root->writing = FALSE;
 
-	return gsf_output_unwrap (G_OBJECT (output), zip->sink);
+	gboolean result = gsf_output_unwrap (G_OBJECT (output), zip->sink);
+
+	/* free unneeded memory */
+	if (zip->stream) {
+		(void) deflateEnd (zip->stream);
+		g_free (zip->stream);
+		zip->stream = NULL;
+		g_free (zip->buf);
+		zip->buf = NULL;
+	}
+
+	return result;
 }
 
 static gboolean
